\name{treeApply}
\alias{treeApply}
\alias{treeNodeDelete}
%- Also NEED an '\alias' for EACH other topic documented here.
\title{ Apply functions to Nodes of a Tree }
\description{
  A list-like object or an unevaluated expression in the language can
  be regarded as a tree structure.  Each element of the object, at
  each level, is a node.  The function \code{treeApply} calls a
  function at each node, with optional variations.
}
\usage{
treeApply(object, expr, env, leafExpr, name = quote(x), preExpr)

treeNodeDelete()
}

\arguments{
  \item{object}{ Any R object. }
  \item{expr}{ Either a function or an unevaluated expression to be
      called on each node.  The calls take place bottom up, that is
      all the elements of each node are dealt with first.  See Details. }
  \item{env}{ The environment in which the calls should take place.
      By default, a new environment is generated under the context of
      the call to \code{treeApply}. }
  \item{leafExpr}{ If supplied, an expression or function to be called
    on the leaf nodes (non-recursive elements).  Defaults to \code{expr}}
  \item{name}{ The variable name used for the current node.  Only
      relevant if a literal expression is supplied rather than a function. }
  \item{preExpr}{ Optional function or expression to be called before
      the recursive computations on the elements.  See Details. }
}
\details{
   Each of \code{expr}, \code{leafExpr}, and \code{preExpr} can be a
   function or the name of a function or a quoted expression, or
   \code{NULL}, the default, meaning no action. If a
   function is supplied, it must take one argument; it  will be called with  the current
   node as that argument, and should return the object that should
   replace the current node in the tree. If an
   expression is supplied, it will involve the R variable specified by
   \code{name},  which will be set to the current node.  The  value of
   the expression will replace the current node in the tree. 
 
  The function \code{treeApply} visits each node, that is the
  \code{object} supplied as the argument, and each element of that
  object or of its elements, examined recursively.  For each node, if
  \code{preExpr} is supplied, that function is called first.  Then
  each element of the current node is visited recursively, and
  replaced by the computed result.  Finally, the node (possibly
  modified) is passed to \code{expr}.  If a node is a leaf  (that is, \emph{not} a
  list-like object) and if \code{leafExpr} was supplied, that is used
  instead of \code{expr}.

  Note that returning a \code{NULL} value does not cause the
  corresponding element to be deleted from the tree.  To signal that a
  node should be deleted, return instead the value of a call to
  \code{treeNodeDelete()}, which exists solely for this purpose.
}
\value{
  The tree made up of the values of \code{expr}, etc. as called on
  each node of \code{object}.
}

\seealso{ \code{\link{lapply}}, which calls a function on each element
of a list, but not recursively.}
\examples{
## a function to work on
ff <- function(obj){
  if(is.matrix(obj) && is.numeric(obj)) return(T)
  if(is.data.frame(obj)) {
    for(v in names(obj))
      {vv <- obj[[v]]; if(is.factor(vv) || !is.numeric(vv)) return(F)}
    T
  }
}
## A function to apply:
## change names T, F to constants TRUE, FALSE
ft <- function(x) { if(identical(x, quote(T))) TRUE
else if(identical(x, quote(F))) FALSE
else x
}
## apply it to the function ff
treeApply(ff, ft)
## More efficient:  only test the leaf nodes
treeApply(ff, leafExpr = ft)
}
\keyword{programming }

